/*
****************************************************************************
*
* MODULE:       d.sites.color
* AUTHOR(S):    Code modified from d.sites
*               Mauro Bailotti, Studio Candidi Tommasi, Udine, ITALY
* PURPOSE:
* COPYRIGHT:    (C) 2001 by the GRASS Development Team
*
*               This program is free software under the GNU General Public
*               License (>=v2). Read the file COPYING that comes with GRASS
*               for details.
*
* METHOD:      d.sites.color displays a GRASS site_lists map using site 
*              markers of size, and type specified by the user. The color 
*              of markers dipends on value in column specified by col 
*              parameter. Output is displayed in the active frame on the 
*              graphics monitor.
*               
*****************************************************************************/

#include "gis.h"
#include "display.h"
#include "D.h"
#include "raster.h"
#define GLOBAL
#include "options.h"
#include "local_proto.h"
#include "site.h"

int main (int argc, char **argv)
{
	char *mapset;
	char msg[200];
	char window_name[64] ;
	int t, b, l, r ;
	struct Cell_head window ;
	struct GModule *module;
	struct Option *opt1, *opt2, *opt3, *opt4, *opt5, *opt6, *opt7, *opt8;
        char *name;
        Site *sitepoint;
        FILE *finput;
        int dims,cat,strs,dbls; //variabili per la descrizione dei sites
        double z,min_z,max_z, diffMaxMin, Passo;
	int col;
	int coltype;

	/* Initialize the GIS calls */
	G_gisinit(argv[0]) ;

	module = G_define_module();
	module->description =
		"Displays site markers in the active display frame "
		"on the graphics monitor.";

	opt4 = G_define_option() ;
	opt4->key        = "sitefile";
	opt4->type       = TYPE_STRING;
	opt4->required   = YES;
	opt4->gisprompt  = "old,site_lists,sites";
	opt4->description= "Name of a site file" ;

	opt2 = G_define_option() ;
	opt2->key        = "size" ;
	opt2->type       = TYPE_INTEGER ;
	opt2->required   = NO ;
	opt2->answer     = "0" ;
	opt2->options    = "0-1000" ;
	opt2->description= "Size, in pixels, in which the icon is to be drawn" ;

	opt3 = G_define_option() ;
	opt3->key        = "type" ;
	opt3->type       = TYPE_STRING ;
	opt3->required   = NO ;
	opt3->answer     = "x" ;
	opt3->options    = "x,diamond,box,+" ;
	opt3->description= "Specify the type of the icon" ;

        opt5 = G_define_option() ;
        opt5->key        = "min" ;
        opt5->type       = TYPE_DOUBLE;
        opt5->required   = NO ;
        opt5->answer     = "0.0" ;
        opt5->description = "Min elevation of sites to plot" ;

        opt6 = G_define_option() ;
        opt6->key        = "max" ;
        opt6->type       = TYPE_DOUBLE;
        opt6->required   = NO ;
        opt6->answer     = "0.0" ;
        opt6->description = "Max elevation of sites to plot" ;

	opt7 = G_define_option() ;
	opt7->key        = "col" ;
        opt7->type       = TYPE_INTEGER;
        opt7->required   = NO ;
        opt7->answer     = "1" ;
        opt7->description = "Column to plot with different colors" ;

	opt8 = G_define_option() ;
	opt8->key        = "coltype" ;
	opt8->type       = TYPE_STRING ;
	opt8->required   = NO ;
	opt8->answer     = "grayscale" ;
	opt8->options    = "grayscale,16color" ;
	opt8->description= "Specify the type of color to use" ;
	
	/* Check command line */

	if (G_parser(argc, argv))
		exit(-1);

	mapset = G_find_file ("site_lists", opt4->answer, "");
	if (mapset == NULL)
	{
		sprintf (msg, "sites file [%s] not found", opt4->answer);
		G_fatal_error (msg);
	}

	infile = G_fopen_sites_old (opt4->answer, mapset);
	if (infile == NULL)
	{
		sprintf (msg, "can't open sites file [%s]", opt4->answer);
		G_fatal_error (msg);
	}
        
        finput = G_sites_open_old(opt4->answer, mapset);

	if (G_site_describe(finput,&dims,&cat,&strs,&dbls)!=0){
	  printf("\nErrore nella lettura del file di ingresso\n");
          exit(-1);
        }

        sitepoint = G_site_new_struct(cat,dims,strs,dbls);

	sscanf(opt2->answer,"%d",&size);
	sscanf(opt7->answer,"%d",&col);
	col = col-1;
	

	if (! strcmp(opt3->answer, "x"))
		type = TYPE_X ;
	else if (! strcmp(opt3->answer, "+"))
		type = TYPE_PLUS ;
	else if (! strcmp(opt3->answer, "box"))
		type = TYPE_BOX ;
	else if (! strcmp(opt3->answer, "diamond"))
		type = TYPE_DIAMOND ;

        if (! strcmp(opt8->answer, "grayscale"))
		coltype = GRAYSCALE ;
	else if (! strcmp(opt8->answer, "16Color"))
		coltype =COLOR ;

	/* Setup driver and check important information */
	if (R_open_driver() != 0)
		G_fatal_error ("No graphics device selected");

	if (D_get_cur_wind(window_name))
		G_fatal_error("No current frame") ;

	if (D_set_cur_wind(window_name))
		G_fatal_error("Current frame not available") ;

	/* Read in the map window associated with window */
	G_get_window(&window) ;

	if (D_check_map_window(&window))
		G_fatal_error("Setting map region") ;

	if (G_set_window(&window) == -1)
		G_fatal_error("Current frame not settable") ;

	/* Determine conversion factors */
	if (D_get_screen_window(&t, &b, &l, &r))
		G_fatal_error("Getting screen frame") ;
	if (D_do_conversions(&window, t, b, l, r))
		G_fatal_error("Error in calculating conversions") ;

	//reads min & max val and if they are not set reads max & min for the col selected in the map
        sscanf(opt5->answer,"%lf",&min_z);
        sscanf(opt6->answer,"%lf",&max_z);
        if (max_z==min_z){
         if (G_site_get(finput,sitepoint)!=-1){
          min_z = sitepoint->dbl_att[col];
          max_z = min_z;
          while (G_site_get(finput,sitepoint)!=-1)
	  {
	   z = sitepoint->dbl_att[col];
           if (z<min_z) min_z=z;
           if (z>max_z) max_z=z;
	  }
         printf("Min= %f \n",min_z);
         printf("Max= %f \n",max_z);
         }
	} else if (max_z<min_z) {
	  G_fatal_error("Max value must be greater than Min value");

	}

        diffMaxMin = max_z - min_z;
        Passo = diffMaxMin / 255;

	if (coltype==GRAYSCALE) {

	switch(type)
	{
	case TYPE_X:
		draw_points_x(&window, min_z, max_z, Passo, col) ;
		break ;
	case TYPE_PLUS:
		draw_points_plus(&window, min_z, max_z, Passo, col) ;
		break ;
	case TYPE_BOX:
		draw_points_box(&window, min_z, max_z, Passo, col) ;
		break ;
	case TYPE_DIAMOND:
		draw_points_diamond(&window, min_z, max_z, Passo, col) ;
		break ;
	}
	} else {
	switch(type)
	{
	case TYPE_X:
		draw_points_x_color(&window, min_z, max_z, Passo, col) ;
		break ;
	case TYPE_PLUS:
		draw_points_plus_color(&window, min_z, max_z, Passo, col) ;
		break ;
	case TYPE_BOX:
		draw_points_box_color(&window, min_z, max_z, Passo, col) ;
		break ;
	case TYPE_DIAMOND:
		draw_points_diamond_color(&window, min_z, max_z, Passo, col) ;
		break ;
	}
	}

	D_add_to_list(G_recreate_command()) ;

	D_set_site_name(G_fully_qualified_name(opt4->answer, mapset));
	D_add_to_site_list(G_fully_qualified_name(opt4->answer, mapset));

	R_close_driver();
	exit(0);
}
